"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var vscode_languageserver_1 = require("vscode-languageserver");
var DiagnosticMessages_1 = require("./DiagnosticMessages");
var BrsFile_1 = require("./files/BrsFile");
var XmlFile_1 = require("./files/XmlFile");
var util_1 = require("./util");
/**
 * A class to keep track of all declarations within a given context (like global scope, component scope)
 */
var Context = /** @class */ (function () {
    function Context(name, matcher) {
        this.name = name;
        this.matcher = matcher;
        /**
         * Indicates whether this context needs to be validated.
         * Will be true when first constructed, or anytime one of its watched files is added, changed, or removed
         */
        this.isValidated = true;
        this.programHandles = [];
        this.parentContextHandles = [];
        this.files = {};
        /**
         * The list of diagnostics found specifically for this context. Individual file diagnostics are stored on the files themselves.
         */
        this.diagnostics = [];
        this.emitter = new events_1.EventEmitter();
        //allow unlimited listeners
        this.emitter.setMaxListeners(0);
    }
    /**
     * Attach the context to a program. This allows the context to monitor file adds, changes, and removals, and respond accordingly
     * @param program
     */
    Context.prototype.attachProgram = function (program) {
        var _this = this;
        this.program = program;
        this.programHandles = [
            program.on('file-added', function (file) {
                if (_this.matcher(file)) {
                    _this.addOrReplaceFile(file);
                }
            }),
            program.on('file-removed', function (file) {
                if (_this.hasFile(file)) {
                    _this.removeFile(file);
                }
            })
        ];
        //add any current matches
        for (var filePath in program.files) {
            var file = program.files[filePath];
            if (this.matcher(file)) {
                this.addOrReplaceFile(file);
            }
        }
    };
    /**
     * Clean up all event handles
     */
    Context.prototype.dispose = function () {
        var e_1, _a;
        try {
            for (var _b = __values(this.programHandles), _c = _b.next(); !_c.done; _c = _b.next()) {
                var disconnect = _c.value;
                disconnect();
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this.detachParent();
    };
    Context.prototype.attachParentContext = function (parent) {
        var _this = this;
        this.parentContext = parent;
        this.parentContextHandles = [
            //whenever the parent is marked dirty, mark ourself as dirty
            parent.on('invalidated', function () {
                _this.isValidated = false;
            })
        ];
        //immediately invalidate self if parent is not validated
        if (!this.parentContext.isValidated) {
            this.isValidated = false;
        }
    };
    Context.prototype.detachParent = function () {
        var e_2, _a;
        try {
            for (var _b = __values(this.parentContextHandles), _c = _b.next(); !_c.done; _c = _b.next()) {
                var disconnect = _c.value;
                disconnect();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        this.parentContext = this.program.platformContext;
    };
    /**
     * Determine if this file should
     * @param filePath
     */
    Context.prototype.shouldIncludeFile = function (file) {
        return this.matcher(file) === true ? true : false;
    };
    /**
     * Get the list of errors for this context. It's calculated on the fly, so
     * call this sparingly.
     */
    Context.prototype.getDiagnostics = function () {
        var diagnosticLists = [this.diagnostics];
        //add diagnostics from every referenced file
        for (var filePath in this.files) {
            var ctxFile = this.files[filePath];
            diagnosticLists.push(ctxFile.file.getDiagnostics());
        }
        var allDiagnostics = Array.prototype.concat.apply([], diagnosticLists);
        var filteredDiagnostics = allDiagnostics.filter(function (x) {
            return !util_1.default.diagnosticIsSuppressed(x);
        });
        //filter out diangostics that match any of the comment flags
        return filteredDiagnostics;
    };
    /**
     * Get the list of callables available in this context (either declared in this context or in a parent context)
     */
    Context.prototype.getAllCallables = function () {
        //get callables from parent contexts
        if (this.parentContext) {
            return __spread(this.getOwnCallables(), this.parentContext.getAllCallables());
        }
        else {
            return __spread(this.getOwnCallables());
        }
    };
    /**
     * Get the callable with the specified name.
     * If there are overridden callables with the same name, the closest callable to this context is returned
     * @param name
     */
    Context.prototype.getCallableByName = function (name) {
        var e_3, _a;
        var lowerName = name.toLowerCase();
        var callables = this.getAllCallables();
        try {
            for (var callables_1 = __values(callables), callables_1_1 = callables_1.next(); !callables_1_1.done; callables_1_1 = callables_1.next()) {
                var callable = callables_1_1.value;
                if (callable.callable.name.toLowerCase() === lowerName) {
                    return callable.callable;
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (callables_1_1 && !callables_1_1.done && (_a = callables_1.return)) _a.call(callables_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    /**
     * Get the list of callables explicitly defined in files in this context.
     * This excludes ancestor callables
     */
    Context.prototype.getOwnCallables = function () {
        var e_4, _a;
        var result = [];
        //get callables from own files
        for (var filePath in this.files) {
            var file = this.files[filePath];
            try {
                for (var _b = __values(file.file.callables), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var callable = _c.value;
                    result.push({
                        callable: callable,
                        context: this
                    });
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_4) throw e_4.error; }
            }
        }
        return result;
    };
    Context.prototype.on = function (eventName, callback) {
        var _this = this;
        this.emitter.on(eventName, callback);
        return function () {
            _this.emitter.removeListener(eventName, callback);
        };
    };
    Context.prototype.emit = function (name, data) {
        this.emitter.emit(name, data);
    };
    /**
     * Add a file to the program.
     * @param filePath
     * @param fileContents
     */
    Context.prototype.addOrReplaceFile = function (file) {
        this.isValidated = false;
        //if the file is already loaded, remove it first
        if (this.files[file.pathAbsolute]) {
            this.removeFile(file);
        }
        var ctxFile = new ContextFile(file);
        //keep a reference to this file
        this.files[file.pathAbsolute] = ctxFile;
    };
    /**
     * Remove the file from this context.
     * If the file doesn't exist, the method exits immediately, but does not throw an error.
     * @param file
     * @param emitRemovedEvent - if false, the 'remove-file' event will not be emitted
     */
    Context.prototype.removeFile = function (file) {
        this.isValidated = false;
        var ctxFile = this.files[file.pathAbsolute];
        if (!ctxFile) {
            return;
        }
        //remove the reference to this file
        delete this.files[file.pathAbsolute];
        this.emit('invalidated');
    };
    Context.prototype.validate = function () {
        //if this context is already validated, no need to revalidate
        if (this.isValidated === true) {
            return;
        }
        //validate our parent before we validate ourself
        if (this.parentContext && this.parentContext.isValidated === false) {
            this.parentContext.validate();
        }
        //clear the context's errors list (we will populate them from this method)
        this.diagnostics = [];
        var callables = this.getAllCallables();
        //sort the callables by filepath and then method name, so the errors will be consistent
        callables = callables.sort(function (a, b) {
            return (
            //sort by path
            a.callable.file.pathAbsolute.localeCompare(b.callable.file.pathAbsolute) ||
                //then sort by method name
                a.callable.name.localeCompare(b.callable.name));
        });
        //get a list of all callables, indexed by their lower case names
        var callableContainersByLowerName = util_1.default.getCallableContainersByLowerName(callables);
        //find all duplicate function declarations
        this.diagnosticFindDuplicateFunctionDeclarations(callableContainersByLowerName);
        //do many per-file checks
        for (var key in this.files) {
            var contextFile = this.files[key];
            this.diagnosticDetectCallsToUnknownFunctions(contextFile.file, callableContainersByLowerName);
            this.diagnosticDetectFunctionCallsWithWrongParamCount(contextFile.file, callableContainersByLowerName);
            this.diagnosticDetectShadowedLocalVars(contextFile.file, callableContainersByLowerName);
        }
        this.isValidated = false;
    };
    /**
     * Detect calls to functions with the incorrect number of parameters
     * @param file
     * @param callableContainersByLowerName
     */
    Context.prototype.diagnosticDetectFunctionCallsWithWrongParamCount = function (file, callableContainersByLowerName) {
        var e_5, _a, e_6, _b;
        try {
            //validate all function calls
            for (var _c = __values(file.functionCalls), _d = _c.next(); !_d.done; _d = _c.next()) {
                var expCall = _d.value;
                var callableContainersWithThisName = callableContainersByLowerName[expCall.name.toLowerCase()];
                //use the first item from callablesByLowerName, because if there are more, that's a separate error
                var knownCallableContainer = callableContainersWithThisName ? callableContainersWithThisName[0] : undefined;
                if (knownCallableContainer) {
                    //get min/max parameter count for callable
                    var minParams = 0;
                    var maxParams = 0;
                    try {
                        for (var _e = __values(knownCallableContainer.callable.params), _f = _e.next(); !_f.done; _f = _e.next()) {
                            var param = _f.value;
                            maxParams++;
                            //optional parameters must come last, so we can assume that minParams won't increase once we hit
                            //the first isOptional
                            if (param.isOptional === false) {
                                minParams++;
                            }
                        }
                    }
                    catch (e_6_1) { e_6 = { error: e_6_1 }; }
                    finally {
                        try {
                            if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                        }
                        finally { if (e_6) throw e_6.error; }
                    }
                    var expCallArgCount = expCall.args.length;
                    if (expCall.args.length > maxParams || expCall.args.length < minParams) {
                        var minMaxParamsText = minParams === maxParams ? maxParams : minParams + '-' + maxParams;
                        this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Expected_a_arguments_but_got_b_1002(minMaxParamsText, expCallArgCount), { location: expCall.nameRange, 
                            //TODO detect end of expression call
                            file: file, severity: 'error' }));
                    }
                }
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    /**
     * Detect local variables (function scope) that have the same name as context calls
     * @param file
     * @param callablesByLowerName
     */
    Context.prototype.diagnosticDetectShadowedLocalVars = function (file, callablesByLowerName) {
        var e_7, _a, e_8, _b;
        try {
            //loop through every function scope
            for (var _c = __values(file.functionScopes), _d = _c.next(); !_d.done; _d = _c.next()) {
                var scope = _d.value;
                try {
                    //every var declaration in this scope
                    for (var _e = __values(scope.variableDeclarations), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var varDeclaration = _f.value;
                        var globalCallableContainer = callablesByLowerName[varDeclaration.name.toLowerCase()];
                        //if we found a collision
                        if (globalCallableContainer && globalCallableContainer.length > 0) {
                            var globalCallable = globalCallableContainer[0];
                            this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Local_var_shadows_global_function_1011(varDeclaration.name, globalCallable.callable.file.pkgPath), { location: varDeclaration.nameRange, file: file, severity: 'warning' }));
                        }
                    }
                }
                catch (e_8_1) { e_8 = { error: e_8_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                    }
                    finally { if (e_8) throw e_8.error; }
                }
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_7) throw e_7.error; }
        }
    };
    /**
     * Detect calls to functions that are not defined in this context
     * @param file
     * @param callablesByLowerName
     */
    Context.prototype.diagnosticDetectCallsToUnknownFunctions = function (file, callablesByLowerName) {
        var e_9, _a;
        try {
            //validate all expression calls
            for (var _b = __values(file.functionCalls), _c = _b.next(); !_c.done; _c = _b.next()) {
                var expCall = _c.value;
                var lowerName = expCall.name.toLowerCase();
                //get the local scope for this expression
                var scope = file.getFunctionScopeAtPosition(expCall.nameRange.start);
                if (scope) {
                    //if we found a variable with the same name as the function, assume the call is "known".
                    //If the variable is a different type, some other check should add a diagnostic for that.
                    if (scope.getVariableByName(lowerName)) {
                        continue;
                    }
                }
                var callablesWithThisName = callablesByLowerName[expCall.name.toLowerCase()];
                //use the first item from callablesByLowerName, because if there are more, that's a separate error
                var knownCallable = callablesWithThisName ? callablesWithThisName[0] : undefined;
                //detect calls to unknown functions
                if (!knownCallable) {
                    this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Call_to_unknown_function_1001(expCall.name, this.name), { location: expCall.nameRange, file: file, severity: 'error' }));
                    continue;
                }
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_9) throw e_9.error; }
        }
    };
    /**
     * Create diagnostics for any duplicate function declarations
     * @param callablesByLowerName
     */
    Context.prototype.diagnosticFindDuplicateFunctionDeclarations = function (callableContainersByLowerName) {
        var e_10, _a, e_11, _b, e_12, _c;
        //for each list of callables with the same name
        for (var lowerName in callableContainersByLowerName) {
            var callableContainers = callableContainersByLowerName[lowerName];
            var platformCallables = [];
            var nonPlatformCallables = [];
            var ownCallables = [];
            var ancestorNonPlatformCallables = [];
            try {
                for (var callableContainers_1 = __values(callableContainers), callableContainers_1_1 = callableContainers_1.next(); !callableContainers_1_1.done; callableContainers_1_1 = callableContainers_1.next()) {
                    var container = callableContainers_1_1.value;
                    if (container.context === this.program.platformContext) {
                        platformCallables.push(container);
                    }
                    else {
                        nonPlatformCallables.push(container);
                        if (container.context === this) {
                            ownCallables.push(container);
                        }
                        else {
                            ancestorNonPlatformCallables.push(container);
                        }
                    }
                }
            }
            catch (e_10_1) { e_10 = { error: e_10_1 }; }
            finally {
                try {
                    if (callableContainers_1_1 && !callableContainers_1_1.done && (_a = callableContainers_1.return)) _a.call(callableContainers_1);
                }
                finally { if (e_10) throw e_10.error; }
            }
            //add info diagnostics about child shadowing parent functions
            if (ownCallables.length > 0 && ancestorNonPlatformCallables.length > 0) {
                try {
                    inner: for (var ownCallables_1 = __values(ownCallables), ownCallables_1_1 = ownCallables_1.next(); !ownCallables_1_1.done; ownCallables_1_1 = ownCallables_1.next()) {
                        var container = ownCallables_1_1.value;
                        //skip the init function (because every component will have one of those){
                        if (lowerName === 'init') {
                            continue inner;
                        }
                        var shadowedCallable = ancestorNonPlatformCallables[ancestorNonPlatformCallables.length - 1];
                        this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Shadows_ancestor_function_1010(container.callable.name, container.context.name, shadowedCallable.callable.file.pkgPath, 
                        //grab the last item in the list, which should be the closest ancestor's version
                        shadowedCallable.context.name), { location: container.callable.nameRange, file: container.callable.file, severity: 'hint' }));
                    }
                }
                catch (e_11_1) { e_11 = { error: e_11_1 }; }
                finally {
                    try {
                        if (ownCallables_1_1 && !ownCallables_1_1.done && (_b = ownCallables_1.return)) _b.call(ownCallables_1);
                    }
                    finally { if (e_11) throw e_11.error; }
                }
            }
            //add error diagnostics about duplicate functions in the same context
            if (ownCallables.length > 1) {
                try {
                    for (var ownCallables_2 = __values(ownCallables), ownCallables_2_1 = ownCallables_2.next(); !ownCallables_2_1.done; ownCallables_2_1 = ownCallables_2.next()) {
                        var callableContainer = ownCallables_2_1.value;
                        var callable = callableContainer.callable;
                        this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Duplicate_function_implementation_1003(callable.name, callableContainer.context.name), { location: vscode_languageserver_1.Range.create(callable.nameRange.start.line, callable.nameRange.start.character, callable.nameRange.start.line, callable.nameRange.end.character), file: callable.file, severity: 'error' }));
                    }
                }
                catch (e_12_1) { e_12 = { error: e_12_1 }; }
                finally {
                    try {
                        if (ownCallables_2_1 && !ownCallables_2_1.done && (_c = ownCallables_2.return)) _c.call(ownCallables_2);
                    }
                    finally { if (e_12) throw e_12.error; }
                }
            }
        }
    };
    /**
     * Find the file with the specified relative path
     * @param relativePath
     */
    Context.prototype.getFileByRelativePath = function (relativePath) {
        for (var key in this.files) {
            if (this.files[key].file.pkgPath.toLowerCase() === relativePath.toLowerCase()) {
                return this.files[key];
            }
        }
    };
    Context.prototype.hasFile = function (file) {
        var pathAbsolute;
        if (file instanceof BrsFile_1.BrsFile || file instanceof XmlFile_1.XmlFile) {
            pathAbsolute = file.pathAbsolute;
        }
        else {
            pathAbsolute = file;
        }
        if (this.files[pathAbsolute]) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * Get all callables as completionItems
     */
    Context.prototype.getCallablesAsCompletions = function () {
        var e_13, _a;
        var completions = [];
        var callables = this.getAllCallables();
        try {
            for (var callables_2 = __values(callables), callables_2_1 = callables_2.next(); !callables_2_1.done; callables_2_1 = callables_2.next()) {
                var callableContainer = callables_2_1.value;
                completions.push({
                    label: callableContainer.callable.name,
                    kind: vscode_languageserver_1.CompletionItemKind.Function,
                    detail: callableContainer.callable.shortDescription,
                    documentation: callableContainer.callable.documentation ? { kind: 'markdown', value: callableContainer.callable.documentation } : undefined
                });
            }
        }
        catch (e_13_1) { e_13 = { error: e_13_1 }; }
        finally {
            try {
                if (callables_2_1 && !callables_2_1.done && (_a = callables_2.return)) _a.call(callables_2);
            }
            finally { if (e_13) throw e_13.error; }
        }
        return completions;
    };
    /**
     * Get the definition (where was this thing first defined) of the symbol under the position
     */
    Context.prototype.getDefinition = function (file, position) {
        //TODO implement for brs files
        return [];
    };
    return Context;
}());
exports.Context = Context;
var ContextFile = /** @class */ (function () {
    function ContextFile(file) {
        this.file = file;
    }
    return ContextFile;
}());
//# sourceMappingURL=Context.js.map