import { InitializeParams, ServerCapabilities } from 'vscode-languageserver';
import { ProgramBuilder } from './ProgramBuilder';
export declare class LanguageServer {
    constructor();
    private connection;
    workspaces: Workspace[];
    private hasConfigurationCapability;
    /**
     * Indicates whether the client supports workspace folders
     */
    private clientHasWorkspaceFolderCapability;
    /**
     * Create a simple text document manager.
     * The text document manager supports full document sync only
     */
    private documents;
    private createConnection;
    run(): void;
    /**
     * Called when the client starts initialization
     * @param params
     */
    onInitialize(params: InitializeParams): Promise<{
        capabilities: ServerCapabilities;
    }>;
    /**
     * Called when the client has finished initializing
     * @param params
     */
    private onInitialized;
    /**
     * Send a critical failure notification to the client, which should show a notification of some kind
     */
    private sendCriticalFailure;
    /**
     * Wait for all programs' first run to complete
     */
    private waitAllProgramFirstRuns;
    /**
     * Create project for each new workspace. If the workspace is already known,
     * it is skipped.
     * @param workspaceFolders
     */
    private createWorkspaces;
    /**
     * Event handler for when the program wants to load file contents.
     * anytime the program wants to load a file, check with our in-memory document cache first
     */
    private documentFileResolver;
    private createWorkspace;
    /**
     * Provide a list of completion items based on the current cursor position
     * @param textDocumentPosition
     */
    private onCompletion;
    /**
     * Provide a full completion item from the selection
     * @param item
     */
    private onCompletionResolve;
    /**
     * Reload all specified workspaces, or all workspaces if no workspaces are specified
     */
    private reloadWorkspaces;
    private onDidChangeConfiguration;
    /**
     * Called when watched files changed (add/change/delete).
     * The client is in charge of what files to watch, so all client
     * implementations should ensure that all valid brightscript project
     * file types are watched (.brs,.bs,.xml,manifest, and any json/text/image files)
     * @param params
     */
    private onDidChangeWatchedFiles;
    private onHover;
    private validateTextDocument;
    private onDefinition;
    /**
     * The list of all issues, indexed by file. This allows us to keep track of which buckets of
     * diagnostics to send and which to skip because nothing has changed
     */
    private latestDiagnosticsByFile;
    private sendDiagnostics;
}
interface Workspace {
    firstRunPromise: Promise<any>;
    builder: ProgramBuilder;
    workspacePath: string;
    isFirstRunComplete: boolean;
    isFirstRunSuccessful: boolean;
    configFilePath?: string;
}
export {};
