import { FileEvent } from 'vscode-languageserver';
import { BrsConfig } from './BrsConfig';
import { FileResolver, Program } from './Program';
/**
 * A runner class that handles
 */
export declare class ProgramBuilder {
    constructor();
    /**
     * Determines whether the console should be cleared after a run (true for cli, false for languageserver)
     */
    allowConsoleClearing: boolean;
    private options;
    private isRunning;
    private watcher;
    program: Program;
    private fileResolvers;
    addFileResolver(fileResolver: FileResolver): void;
    /**
     * The list of errors found in the program.
     */
    private getDiagnostics;
    run(options: BrsConfig): Promise<void>;
    private clearConsole;
    enableWatchMode(): Promise<void>;
    /**
     * A method that is used to cancel a previous run task.
     * Does nothing if previous run has completed or was already canceled
     */
    private cancelLastRun;
    /**
     * Run the entire process exactly one time.
     */
    private runOnce;
    private printDiagnostics;
    /**
     * Run the process once, allowing cancelability.
     * NOTE: This should only be called by `runOnce`.
     * @param cancellationToken
     */
    private _runOnce;
    private createPackageIfEnabled;
    private deployPackageIfEnabled;
    /**
     * Parse and load the AST for every file in the project
     */
    private loadAllFilesAST;
    /**
     * This only operates on files that match the specified files globs, so it is safe to throw
     * any file changes you receive with no unexpected side-effects
     * @param changes
     */
    handleFileChanges(changes: FileEvent[]): Promise<void>;
    /**
     * Remove all files from the program that are in the specified folder path
     * @param folderPathAbsolute
     */
    removeFilesInFolder(folderPathAbsolute: string): Promise<void>;
    /**
     * Load any files from the given folder that are not already loaded into the program.
     * This is mainly used when folders get moved, but we also have some active changes in
     * some of the files from the new location already
     * @param folderPathAbsolute
     */
    loadMissingFilesFromFolder(folderPathAbsolute: string): Promise<void>;
    /**
     * Scan every file and resolve all variable references.
     * If no errors were encountered, return true. Otherwise return false.
     */
    private validateProject;
    dispose(): void;
}
