"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var vscode_languageserver_1 = require("vscode-languageserver");
var Context_1 = require("./Context");
var DiagnosticMessages_1 = require("./DiagnosticMessages");
var XmlFile_1 = require("./files/XmlFile");
var util_1 = require("./util");
var XmlContext = /** @class */ (function (_super) {
    __extends(XmlContext, _super);
    function XmlContext(xmlFile) {
        var _this = _super.call(this, xmlFile.pkgPath, function (file) {
            return _this.xmlFile.doesReferenceFile(file);
        }) || this;
        _this.xmlFileHandles = [];
        _this.xmlFile = xmlFile;
        _this.xmlFileHandles = [
            //when the xml file gets a parent added, link to that parent's context
            _this.xmlFile.on('attach-parent', function (parent) {
                _this.handleXmlFileParentAttach(parent);
            }),
            //when xml file detaches its parent, remove the context link
            _this.xmlFile.on('detach-parent', function () {
                _this.detachParent();
            }),
        ];
        //if the xml file already has a parent attached, attach our context to that parent xml's context
        if (_this.xmlFile.parent) {
            _this.handleXmlFileParentAttach(_this.xmlFile.parent);
        }
        return _this;
    }
    XmlContext.prototype.attachProgram = function (program) {
        var _this = this;
        _super.prototype.attachProgram.call(this, program);
        //if the xml file has an unresolved parent, look for its parent on every file add
        this.programHandles.push(this.program.on('file-added', function (file) {
            _this.addParentIfMatch(file);
        }));
        //detach xml file's parent if it's removed from the program
        this.programHandles.push(this.program.on('file-removed', this.onProgramFileRemove.bind(this)));
        //try finding and attaching the parent component
        for (var key in this.program.files) {
            this.addParentIfMatch(this.program.files[key]);
        }
        //if the xml file already has a parent xml file, attach it
        if (this.xmlFile.parent && this.xmlFile.parent !== this.program.platformContext) {
            this.handleXmlFileParentAttach(this.xmlFile.parent);
        }
    };
    /**
     * Event handler for when the attached program removes a file
     * @param file
     */
    XmlContext.prototype.onProgramFileRemove = function (file) {
        if (
        //incoming file is an xml file
        file instanceof XmlFile_1.XmlFile &&
            //incoming file has a component name
            file.componentName &&
            //this xml file has a parent
            this.xmlFile.parentName &&
            //incoming file has same name as parent
            file.componentName.toLowerCase() === this.xmlFile.parentName.toLowerCase()) {
            this.isValidated = false;
            this.xmlFile.detachParent();
            //disconnect the context link
            this.detachParent();
        }
    };
    XmlContext.prototype.handleXmlFileParentAttach = function (file) {
        var parentContext = this.program.contexts[file.pkgPath];
        if (parentContext) {
            this.attachParentContext(parentContext);
        }
    };
    XmlContext.prototype.addParentIfMatch = function (file) {
        if (
        //xml file has no parent
        !this.xmlFile.parent &&
            //xml file WANTS a parent
            this.xmlFile.parentName &&
            //incoming file is an xml file
            file instanceof XmlFile_1.XmlFile &&
            //xml file's name matches the desired parent name
            file.componentName === this.xmlFile.parentName) {
            this.isValidated = false;
            this.xmlFile.attachParent(file);
        }
    };
    XmlContext.prototype.validate = function () {
        if (this.isValidated === false) {
            _super.prototype.validate.call(this);
            //detect when the child imports a script that its ancestor also imports
            this.diagnosticDetectDuplicateAncestorScriptImports();
            //detect script imports to files that are not loaded in this context
            this.diagnosticValidateScriptImportPaths();
        }
    };
    /**
     * Detect when a child has imported a script that an ancestor also imported
     */
    XmlContext.prototype.diagnosticDetectDuplicateAncestorScriptImports = function () {
        var e_1, _a, e_2, _b;
        if (this.xmlFile.parent) {
            //build a lookup of pkg paths -> FileReference so we can more easily look up collisions
            var parentScriptImports = this.xmlFile.parent.getAllScriptImports();
            var lookup = {};
            try {
                for (var parentScriptImports_1 = __values(parentScriptImports), parentScriptImports_1_1 = parentScriptImports_1.next(); !parentScriptImports_1_1.done; parentScriptImports_1_1 = parentScriptImports_1.next()) {
                    var parentScriptImport = parentScriptImports_1_1.value;
                    //keep the first occurance of a pkgPath. Parent imports are first in the array
                    if (!lookup[parentScriptImport.pkgPath]) {
                        lookup[parentScriptImport.pkgPath] = parentScriptImport;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (parentScriptImports_1_1 && !parentScriptImports_1_1.done && (_a = parentScriptImports_1.return)) _a.call(parentScriptImports_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                //add warning for every import this file shares with an ancestor
                for (var _c = __values(this.xmlFile.ownScriptImports), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var scriptImport = _d.value;
                    var ancestorScriptImport = lookup[scriptImport.pkgPath];
                    if (ancestorScriptImport) {
                        var ancestorComponentName = ancestorScriptImport.sourceFile.componentName;
                        this.diagnostics.push(__assign({ severity: 'warning', file: this.xmlFile, location: vscode_languageserver_1.Range.create(scriptImport.lineIndex, scriptImport.columnIndexBegin, scriptImport.lineIndex, scriptImport.columnIndexEnd) }, DiagnosticMessages_1.diagnosticMessages.Unnecessary_script_import_in_child_from_parent_1009(ancestorComponentName)));
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return)) _b.call(_c);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
    };
    /**
     * Get the definition (where was this thing first defined) of the symbol under the position
     */
    XmlContext.prototype.getDefinition = function (file, position) {
        var results = [];
        //if the position is within the file's parent component name
        if (file instanceof XmlFile_1.XmlFile &&
            file.parent &&
            file.parentNameRange &&
            util_1.default.rangeContains(file.parentNameRange, position)) {
            results.push({
                range: vscode_languageserver_1.Range.create(0, 0, 0, 0),
                uri: util_1.default.pathToUri(file.parent.pathAbsolute)
            });
        }
        return results;
    };
    /**
     * Verify that all of the scripts ipmorted by
     */
    XmlContext.prototype.diagnosticValidateScriptImportPaths = function () {
        var e_3, _a;
        try {
            //verify every script import
            for (var _b = __values(this.xmlFile.ownScriptImports), _c = _b.next(); !_c.done; _c = _b.next()) {
                var scriptImport = _c.value;
                var referencedFile = this.getFileByRelativePath(scriptImport.pkgPath);
                //if we can't find the file
                if (!referencedFile) {
                    var message = void 0;
                    var code = void 0;
                    if (scriptImport.text.trim().length === 0) {
                        message = DiagnosticMessages_1.diagnosticMessages.Script_src_cannot_be_empty_1015().message;
                        code = DiagnosticMessages_1.diagnosticMessages.Script_src_cannot_be_empty_1015().code;
                    }
                    else {
                        message = DiagnosticMessages_1.diagnosticMessages.Referenced_file_does_not_exist_1004().message;
                        code = DiagnosticMessages_1.diagnosticMessages.Referenced_file_does_not_exist_1004().code;
                    }
                    this.diagnostics.push({
                        message: message,
                        code: code,
                        location: vscode_languageserver_1.Range.create(scriptImport.lineIndex, scriptImport.columnIndexBegin, scriptImport.lineIndex, scriptImport.columnIndexEnd),
                        file: this.xmlFile,
                        severity: 'error',
                    });
                }
                else {
                    //if the character casing of the script import path does not match that of the actual path
                    if (scriptImport.pkgPath !== referencedFile.file.pkgPath) {
                        this.diagnostics.push(__assign({}, DiagnosticMessages_1.diagnosticMessages.Script_import_case_mismatch_1012(referencedFile.file.pkgPath), { location: vscode_languageserver_1.Range.create(scriptImport.lineIndex, scriptImport.columnIndexBegin, scriptImport.lineIndex, scriptImport.columnIndexEnd), file: this.xmlFile, severity: 'warning' }));
                    }
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    XmlContext.prototype.dispose = function () {
        var e_4, _a;
        _super.prototype.dispose.call(this);
        try {
            for (var _b = __values(this.xmlFileHandles), _c = _b.next(); !_c.done; _c = _b.next()) {
                var disconnect = _c.value;
                disconnect();
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
    };
    return XmlContext;
}(Context_1.Context));
exports.XmlContext = XmlContext;
//# sourceMappingURL=XmlContext.js.map