import * as brs from 'brs';
import { CompletionItem, Hover, Position } from 'vscode-languageserver';
import { Context } from '../Context';
import { FunctionScope } from '../FunctionScope';
import { Callable, CommentFlag, Diagnostic, FunctionCall } from '../interfaces';
import { Program } from '../Program';
/**
 * Holds all details about this file within the context of the whole program
 */
export declare class BrsFile {
    pathAbsolute: string;
    /**
     * The absolute path to the file, relative to the pkg
     */
    pkgPath: string;
    program: Program;
    constructor(pathAbsolute: string, 
    /**
     * The absolute path to the file, relative to the pkg
     */
    pkgPath: string, program: Program);
    /**
     * The extension for this file
     */
    extension: string;
    /**
     * Indicates if this file was processed by the program yet.
     */
    wasProcessed: boolean;
    private diagnostics;
    getDiagnostics(): Diagnostic[];
    commentFlags: CommentFlag[];
    callables: Callable[];
    functionCalls: FunctionCall[];
    functionScopes: FunctionScope[];
    /**
     * The AST for this file
     */
    private ast;
    private tokens;
    /**
     * Get the token at the specified position
     * @param position
     */
    private getTokenAt;
    /**
     * Calculate the AST for this file
     * @param fileContents
     */
    parse(fileContents: string): Promise<void>;
    standardizeLexParseErrors(errors: brs.parser.ParseError[], lines: string[]): Diagnostic[];
    /**
     * Find all comment flags in the source code. These enable or disable diagnostic messages.
     * @param lines - the lines of the program
     */
    getIgnores(lines: string[]): void;
    scopesByFunc: Map<brs.parser.Expr.Function, FunctionScope>;
    /**
     * Create a scope for every function in this file
     */
    private createFunctionScopes;
    /**
     * Given a set of statements and top-level ast,
     * find the closest function ancestor for the given key
     * @param statements
     * @param key
     */
    private getAncestors;
    private getBRSTypeFromAssignment;
    private getCallableByName;
    private findCallables;
    private findFunctionCalls;
    /**
     * Find the function scope at the given position.
     * @param position
     * @param functionScopes
     */
    getFunctionScopeAtPosition(position: Position, functionScopes?: FunctionScope[]): FunctionScope;
    getCompletions(position: Position, context?: Context): {
        completions: CompletionItem[];
        includeContextCallables: boolean;
    };
    getHover(position: Position): Hover;
    dispose(): void;
}
