"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var sinonImport = require("sinon");
var vscode_languageserver_1 = require("vscode-languageserver");
var Program_1 = require("../Program");
var BooleanType_1 = require("../types/BooleanType");
var DynamicType_1 = require("../types/DynamicType");
var FunctionType_1 = require("../types/FunctionType");
var IntegerType_1 = require("../types/IntegerType");
var StringType_1 = require("../types/StringType");
var BrsFile_1 = require("./BrsFile");
var sinon = sinonImport.createSandbox();
describe('BrsFile', function () {
    var rootDir = process.cwd();
    var program;
    var file;
    beforeEach(function () {
        program = new Program_1.Program({ rootDir: rootDir });
        file = new BrsFile_1.BrsFile('abs', 'rel', program);
    });
    afterEach(function () {
        sinon.restore();
    });
    describe('comment flags', function () {
        describe('brs:disable-next-line', function () {
            it('works for all', function () { return __awaiter(_this, void 0, void 0, function () {
                var file;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub Main()\n                        'brs:disable-next-line\n                        name = \"bob\n                    end sub\n                ")];
                        case 1:
                            file = _a.sent();
                            chai_1.expect(file.commentFlags[0]).to.exist;
                            chai_1.expect(file.commentFlags[0]).to.deep.include({
                                codes: null,
                                range: vscode_languageserver_1.Range.create(2, 24, 2, 46),
                                affectedRange: vscode_languageserver_1.Range.create(3, 0, 3, 35)
                            });
                            return [4 /*yield*/, program.validate()];
                        case 2:
                            _a.sent();
                            //the "unterminated string" error should be filtered out
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(0);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('works for specific codes', function () { return __awaiter(_this, void 0, void 0, function () {
                var file;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub Main()\n                        'brs:disable-next-line: 1000, 1001\n                        name = \"bob\n                    end sub\n                ")];
                        case 1:
                            file = _a.sent();
                            chai_1.expect(file.commentFlags[0]).to.exist;
                            chai_1.expect(file.commentFlags[0]).to.deep.include({
                                codes: [1000, 1001],
                                range: vscode_languageserver_1.Range.create(2, 24, 2, 58),
                                affectedRange: vscode_languageserver_1.Range.create(3, 0, 3, 35)
                            });
                            //the "unterminated string" error should be filtered out
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(0);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('adds diagnostics for unknown diagnostic codes', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub main()\n                        print \"hi\" 'brs:disable-line: 123456 999999   aaaab\n                    end sub\n                ")];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, program.validate()];
                        case 2:
                            _a.sent();
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(3);
                            chai_1.expect(program.getDiagnostics()[0]).to.deep.include({
                                location: vscode_languageserver_1.Range.create(2, 54, 2, 60)
                            });
                            chai_1.expect(program.getDiagnostics()[1]).to.deep.include({
                                location: vscode_languageserver_1.Range.create(2, 61, 2, 67)
                            });
                            chai_1.expect(program.getDiagnostics()[2]).to.deep.include({
                                location: vscode_languageserver_1.Range.create(2, 70, 2, 75)
                            });
                            return [2 /*return*/];
                    }
                });
            }); });
        });
        describe('brs:disable-line', function () {
            it('works for all', function () { return __awaiter(_this, void 0, void 0, function () {
                var file;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub Main()\n                        name = \"bob 'brs:disable-line\n                    end sub\n                ")];
                        case 1:
                            file = _a.sent();
                            chai_1.expect(file.commentFlags[0]).to.exist;
                            chai_1.expect(file.commentFlags[0]).to.deep.include({
                                codes: null,
                                range: vscode_languageserver_1.Range.create(2, 36, 2, 53),
                                affectedRange: vscode_languageserver_1.Range.create(2, 0, 2, 36)
                            });
                            return [4 /*yield*/, program.validate()];
                        case 2:
                            _a.sent();
                            //the "unterminated string" error should be filtered out
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(0);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('works for specific codes', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub main()\n                        'should not have any errors\n                        DoSomething(1) 'brs:disable-line:1002\n                        'should have an error because the param-count error is not being suppressed\n                        DoSomething(1) 'brs:disable-line:1000\n                    end sub\n                    sub DoSomething()\n                    end sub\n                ")];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, program.validate()];
                        case 2:
                            _a.sent();
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(1);
                            chai_1.expect(program.getDiagnostics()[0]).to.deep.include({
                                location: vscode_languageserver_1.Range.create(5, 24, 5, 35)
                            });
                            return [2 /*return*/];
                    }
                });
            }); });
            it('handles the erraneous `stop` keyword', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: 
                        //the current version of BRS causes parse errors after the `parse` keyword, showing error in comments
                        //the program should ignore all diagnostics found in brs:* comment lines EXCEPT
                        //for the diagnostics about using unknown error codes
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                    sub main()\n                        stop 'brs:disable-line\n                        print \"need a valid line to fix stop error\"\n                    end sub\n                ")];
                        case 1:
                            //the current version of BRS causes parse errors after the `parse` keyword, showing error in comments
                            //the program should ignore all diagnostics found in brs:* comment lines EXCEPT
                            //for the diagnostics about using unknown error codes
                            _a.sent();
                            return [4 /*yield*/, program.validate()];
                        case 2:
                            _a.sent();
                            chai_1.expect(program.getDiagnostics()).to.be.lengthOf(0);
                            return [2 /*return*/];
                    }
                });
            }); });
        });
    });
    describe('parse', function () {
        it('supports labels and goto statements', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    'multiple goto statements on one line\n                    goto myLabel : goto myLabel\n                    myLabel:\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports empty print statements', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                   print\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports single-word #elseif and #endif', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                    #const someFlag = true\n                    #if someFlag\n                        'code to execute when someFlag is true\n                    #elseif someFlag\n                        'code to execute when anotherFlag is true\n                    #endif\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        console.log(file.getDiagnostics());
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports multi-word #else if and #end if', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                    #const someFlag = true\n                    #if someFlag\n                        'code to execute when someFlag is true\n                    #else if someFlag\n                        'code to execute when anotherFlag is true\n                    #end if\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        console.log(file.getDiagnostics());
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not choke on invalid code inside a false conditional compile', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                    #if false\n                        non-commented code here should not cause parse errors\n                    #end if\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        console.log(file.getDiagnostics());
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports stop statement', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                   stop\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports single-line if statements', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub main()\n                    if 1 < 2: return true: end if\n                    if 1 < 2: return true\n                    end if\n                    if false : print \"true\" : end if\n                    if true: print \"8 worked\": else if true: print \"not run\": else: print \"not run\": end if\n                    if true then : test = sub() : print \"yes\" : end sub : end if\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports line_num as global variable', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    print LINE_NUM\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports many keywords as object property names', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    person = {}\n                    person.and = true\n                    person.box = true\n                    person.createobject = true\n                    person.dim = true\n                    person.double = true\n                    person.each = true\n                    person.else = true\n                    person.elseif = true\n                    person.end = true\n                    person.endfor = true\n                    person.endfunction = true\n                    person.endif = true\n                    person.endsub = true\n                    person.endwhile = true\n                    person.eval = true\n                    person.exit = true\n                    person.exitfor = true\n                    person.exitwhile = true\n                    person.false = true\n                    person.float = true\n                    person.for = true\n                    person.foreach = true\n                    person.function = true\n                    person.getglobalaa = true\n                    person.getlastruncompileerror = true\n                    person.getlastrunruntimeerror = true\n                    person.goto = true\n                    person.if = true\n                    person.integer = true\n                    person.invalid = true\n                    person.let = true\n                    person.line_num = true\n                    person.longinteger = true\n                    person.next = true\n                    person.not = true\n                    person.objfun = true\n                    person.or = true\n                    person.pos = true\n                    person.print = true\n                    person.rem = true\n                    person.return = true\n                    person.run = true\n                    person.step = true\n                    person.stop = true\n                    person.string = true\n                    person.sub = true\n                    person.tab = true\n                    person.then = true\n                    person.to = true\n                    person.true = true\n                    person.type = true\n                    person.while = true\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error on numeric literal type designators', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                    print &he2\n                    print 1.2E+2\n                    print 2!\n                    print 12D-12\n                    print 2.3#\n                    print &hFEDCBA9876543210&\n                    print 9876543210&\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error when encountering sub with return type', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main() as integer\n                    return\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        console.log(file.getDiagnostics());
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not lose function scopes when mismatched end sub', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                    sayHi()\n                end function\n\n                sub sayHi()\n                    print \"hello world\"\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.functionScopes).to.be.lengthOf(2);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not lose sub scope when mismatched end function', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function main()\n                    sayHi()\n                end sub\n\n                sub sayHi()\n                    print \"hello world\"\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.functionScopes).to.be.lengthOf(2);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error with boolean in RHS of set statement', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                    foo = {\n                        bar: false\n                    }\n                    foo.bar = true and false or 3 > 4\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error with boolean in RHS of set statement', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                    m = {\n                        isTrue: false\n                    }\n                    m.isTrue = true = true\n                    m.isTrue = m.isTrue = true\n                    m.isTrue = m.isTrue = m.isTrue\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports variable names ending with type designators', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                  name$ = \"bob\"\n                  age% = 1\n                  height! = 5.5\n                  salary# = 9.87654321\n                  someHex& = 13\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports multiple spaces between two-word keywords', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub main()\n                    if true then\n                        print \"true\"\n                    else    if true then\n                        print \"also true\"\n                    end if\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error with `stop` as object key', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function GetObject()\n                    obj = {\n                        stop: function() as void\n\n                        end function\n                    }\n                    return obj\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('does not error with `run` as object key', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function GetObject()\n                    obj = {\n                        run: function() as void\n\n                        end function\n                    }\n                    return obj\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports assignment operators', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    x = 1\n                    x += 1\n                    x += 2\n                    x -= 1\n                    x /= 2\n                    x = 9\n                    x \\= 2\n                    x *= 3.0\n                    x -= 1\n                    print x\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports `then` as object property', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    promise = {\n                        then: sub()\n                        end sub\n                    }\n                    promise.then()\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports function as parameter type', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    doWork = function(callback as function)\n                    end function\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports increment operator', function () { return __awaiter(_this, void 0, void 0, function () {
            var diagnostics;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    x = 3\n                    x++\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        diagnostics = file.getDiagnostics();
                        if (diagnostics.length > 0) {
                            console.log(diagnostics);
                        }
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports decrement operator', function () { return __awaiter(_this, void 0, void 0, function () {
            var diagnostics;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    x = 3\n                    x--\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        diagnostics = file.getDiagnostics();
                        if (diagnostics.length > 0) {
                            console.log(diagnostics);
                        }
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports writing numbers with decimal but no trailing digit', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    x = 3.\n                    print x\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports assignment operators against object properties', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    m.age = 1\n\n                    m.age += 1\n                    m.age -= 1\n                    m.age *= 1\n                    m.age /= 1\n                    m.age \\= 1\n\n                    m[\"age\"] += 1\n                    m[\"age\"] -= 1\n                    m[\"age\"] *= 1\n                    m[\"age\"] /= 1\n                    m[\"age\"] \\= 1\n\n                    print m.age\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        //skipped until `brs` supports this
        it('supports bitshift assignment operators', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    x = 1\n                    x <<= 8\n                    x >>= 4\n                    print x\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        //skipped until `brs` supports this
        it('supports bitshift assignment operators on objects', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                    function Main()\n                        m.x = 1\n                        m.x <<= 1\n                        m.x >>= 1\n                        print m.x\n                    end function\n                ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports leading and trailing periods for numeric literals', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    one = 1.\n                    print one\n                    pointOne = .1\n                    print pointOne\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it.skip('supports bitshift assignment operators on object properties accessed by array syntax', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                    function Main()\n                        m.x = 1\n                        'm['x'] << 1\n                        'm['x'] >> 1\n                        print m.x\n                    end function\n                ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it.skip('supports weird period AA accessor', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Main()\n                    m._uuid = \"123\"\n                    print m.[\"_uuid\"]\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports library imports', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                Library \"v30/bslCore.brs\"\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports colons as separators in associative array properties', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    obj = {x:0 : y: 1}\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('succeeds when finding variables with "sub" in them', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function DoSomething()\n                    return value.subType()\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.callables[0]).to.deep.include({
                            bodyRange: vscode_languageserver_1.Range.create(2, 0, 3, 16),
                            file: file,
                            nameRange: vscode_languageserver_1.Range.create(1, 25, 1, 36)
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('succeeds when finding variables with the word "function" in them', function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, file.parse("\n                function Test()\n                    typeCheckFunction = RBS_CMN_GetFunction(invalid, methodName)\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds line and column numbers for functions', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function DoA()\n                    print \"A\"\n                end function\n\n                 function DoB()\n                     print \"B\"\n                 end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.callables[0].name).to.equal('DoA');
                        chai_1.expect(file.callables[0].nameRange).to.eql(vscode_languageserver_1.Range.create(1, 25, 1, 28));
                        chai_1.expect(file.callables[1].name).to.equal('DoB');
                        chai_1.expect(file.callables[1].nameRange).to.eql(vscode_languageserver_1.Range.create(5, 26, 5, 29));
                        return [2 /*return*/];
                }
            });
        }); });
        it('throws an error if the file has already been parsed', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('abspath', 'relpath', program);
                        return [4 /*yield*/, file.parse("'a comment")];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, file.parse("'a new comment")];
                    case 3:
                        _a.sent();
                        chai_1.assert.fail(null, null, 'Should have thrown an exception, but did not');
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _a.sent();
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        }); });
        it('finds and registers duplicate callables', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function DoA()\n                    print \"A\"\n                end function\n\n                 function DoA()\n                     print \"A\"\n                 end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.callables.length).to.equal(2);
                        chai_1.expect(file.callables[0].name).to.equal('DoA');
                        chai_1.expect(file.callables[0].nameRange.start.line).to.equal(1);
                        chai_1.expect(file.callables[1].name).to.equal('DoA');
                        chai_1.expect(file.callables[1].nameRange.start.line).to.equal(5);
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds function call line and column numbers', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function DoA()\n                    DoB(\"a\")\n                end function\n                function DoB(a as string)\n                    DoC()\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.functionCalls.length).to.equal(2);
                        chai_1.expect(file.functionCalls[0].range).to.eql(vscode_languageserver_1.Range.create(2, 20, 2, 28));
                        chai_1.expect(file.functionCalls[0].nameRange).to.eql(vscode_languageserver_1.Range.create(2, 20, 2, 23));
                        chai_1.expect(file.functionCalls[1].range).to.eql(vscode_languageserver_1.Range.create(5, 20, 5, 25));
                        chai_1.expect(file.functionCalls[1].nameRange).to.eql(vscode_languageserver_1.Range.create(5, 20, 5, 23));
                        return [2 /*return*/];
                }
            });
        }); });
        it('sanitizes brs errors', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function DoSomething\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics().length).to.be.greaterThan(0);
                        chai_1.expect(file.getDiagnostics()[0]).to.deep.include({
                            file: file
                        });
                        chai_1.expect(file.getDiagnostics()[0].location.start.line).to.equal(1);
                        return [2 /*return*/];
                }
            });
        }); });
        it('supports using the `next` keyword in a for loop', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                sub countit()\n                    for each num in [1,2,3]\n                        print num\n                    next\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        console.error(file.getDiagnostics());
                        chai_1.expect(file.getDiagnostics()).to.be.empty;
                        return [2 /*return*/];
                }
            });
        }); });
        //test is not working yet, but will be enabled when brs supports this syntax
        it('supports assigning functions to objects', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function main()\n                    o = CreateObject(\"roAssociativeArray\")\n                    o.sayHello = sub()\n                        print \"hello\"\n                    end sub\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getDiagnostics().length).to.equal(0);
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('findCallables', function () {
        it('finds body range', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, callable;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                sub Sum()\n                    print \"hello world\"\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        callable = file.callables[0];
                        chai_1.expect(callable.bodyRange).to.eql(vscode_languageserver_1.Range.create(2, 0, 3, 16));
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds correct body range even with inner function', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, callable;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                sub Sum()\n                    sayHi = sub()\n                        print \"Hi\"\n                    end sub\n                    sayHi()\n                end sub\n            ")];
                    case 1:
                        _a.sent();
                        callable = file.callables[0];
                        chai_1.expect(callable.bodyRange).to.eql(vscode_languageserver_1.Range.create(2, 0, 6, 16));
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds callable parameters', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, callable;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function Sum(a, b, c)\n\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        callable = file.callables[0];
                        chai_1.expect(callable.params[0]).to.deep.include({
                            name: 'a',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[0].type).instanceof(DynamicType_1.DynamicType);
                        chai_1.expect(callable.params[1]).to.deep.include({
                            name: 'b',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[1].type).instanceof(DynamicType_1.DynamicType);
                        chai_1.expect(callable.params[2]).to.deep.include({
                            name: 'c',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[2].type).instanceof(DynamicType_1.DynamicType);
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds optional parameters', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, callable;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function Sum(a=2)\n\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        callable = file.callables[0];
                        chai_1.expect(callable.params[0]).to.deep.include({
                            name: 'a',
                            isOptional: true,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[0].type).instanceof(DynamicType_1.DynamicType);
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds parameter types', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, callable;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function Sum(a, b as integer, c as string)\n\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        callable = file.callables[0];
                        chai_1.expect(callable.params[0]).to.deep.include({
                            name: 'a',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[0].type).instanceof(DynamicType_1.DynamicType);
                        chai_1.expect(callable.params[1]).to.deep.include({
                            name: 'b',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[1].type).instanceof(IntegerType_1.IntegerType);
                        chai_1.expect(callable.params[2]).to.deep.include({
                            name: 'c',
                            isOptional: false,
                            isRestArgument: false
                        });
                        chai_1.expect(callable.params[2].type).instanceof(StringType_1.StringType);
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('findCallableInvocations', function () {
        it('finds arguments with literal values', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, args;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function Sum()\n                    DoSomething(\"name\", 12, true)\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.functionCalls.length).to.equal(1);
                        args = file.functionCalls[0].args;
                        chai_1.expect(args.length).to.equal(3);
                        chai_1.expect(args[0]).deep.include({
                            type: new StringType_1.StringType(),
                            text: '"name"'
                        });
                        chai_1.expect(args[1]).deep.include({
                            type: new IntegerType_1.IntegerType(),
                            text: '12'
                        });
                        chai_1.expect(args[2]).deep.include({
                            type: new BooleanType_1.BooleanType(),
                            text: 'true'
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds arguments with variable values', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute_path/file.brs', 'relative_path/file.brs', program);
                        return [4 /*yield*/, file.parse("\n                function Sum()\n                    count = 1\n                    name = \"John\"\n                    isAlive = true\n                    DoSomething(count, name, isAlive)\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.functionCalls.length).to.equal(1);
                        chai_1.expect(file.functionCalls[0].args[0]).deep.include({
                            type: new DynamicType_1.DynamicType(),
                            text: 'count'
                        });
                        chai_1.expect(file.functionCalls[0].args[1]).deep.include({
                            type: new DynamicType_1.DynamicType(),
                            text: 'name'
                        });
                        chai_1.expect(file.functionCalls[0].args[2]).deep.include({
                            type: new DynamicType_1.DynamicType(),
                            text: 'isAlive'
                        });
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('standardizeLexParserErrors', function () {
        it('properly maps the location to a Range', function () {
            var file = new BrsFile_1.BrsFile('', '', program);
            chai_1.expect(file.standardizeLexParseErrors([{
                    location: {
                        start: {
                            column: 0,
                            line: 1
                        }, end: {
                            column: 4,
                            line: 2
                        },
                        file: ''
                    },
                    message: 'some lex error',
                    stack: ''
                }], [])).to.eql([{
                    code: 1000,
                    message: 'some lex error',
                    location: vscode_languageserver_1.Range.create(0, 0, 1, 4),
                    file: file,
                    severity: 'error'
                }]);
        });
    });
    describe('findCallables', function () {
        //this test is to help with code coverage
        it('skips top-level statements', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute', 'relative', program);
                        return [4 /*yield*/, file.parse('name = "Bob"')];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.callables.length).to.equal(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds return type', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new BrsFile_1.BrsFile('absolute', 'relative', program);
                        return [4 /*yield*/, file.parse("\n                function DoSomething() as string\n                end function\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.callables[0]).to.deep.include({
                            file: file,
                            nameRange: vscode_languageserver_1.Range.create(1, 25, 1, 36),
                            name: 'DoSomething',
                            params: []
                        });
                        chai_1.expect(file.callables[0].type.returnType).instanceof(StringType_1.StringType);
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('createFunctionScopes', function () { return __awaiter(_this, void 0, void 0, function () {
        var _this = this;
        return __generator(this, function (_a) {
            it('creates range properly', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    name = 'bob\"\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes[0].range).to.eql(vscode_languageserver_1.Range.create(1, 16, 3, 23));
                            return [2 /*return*/];
                    }
                });
            }); });
            it('creates scopes for parent and child functions', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    sayHi = sub()\n                        print \"hi\"\n                    end sub\n\n                    scheduleJob(sub()\n                        print \"job completed\"\n                    end sub)\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes).to.length(3);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('outer function does not capture inner statements', function () { return __awaiter(_this, void 0, void 0, function () {
                var outerScope, innerScope;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    name = \"john\"\n                    sayHi = sub()\n                        age = 12\n                    end sub\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            outerScope = file.getFunctionScopeAtPosition(vscode_languageserver_1.Position.create(2, 25));
                            chai_1.expect(outerScope.variableDeclarations).to.be.lengthOf(2);
                            innerScope = file.getFunctionScopeAtPosition(vscode_languageserver_1.Position.create(4, 10));
                            chai_1.expect(innerScope.variableDeclarations).to.be.lengthOf(1);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('finds variables declared in function scopes', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    sayHi = sub()\n                        age = 12\n                    end sub\n\n                    scheduleJob(sub()\n                        name = \"bob\"\n                    end sub)\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes[0].variableDeclarations).to.be.length(1);
                            chai_1.expect(file.functionScopes[0].variableDeclarations[0]).to.deep.include({
                                lineIndex: 2,
                                name: 'sayHi'
                            });
                            chai_1.expect(file.functionScopes[0].variableDeclarations[0].type).instanceof(FunctionType_1.FunctionType);
                            chai_1.expect(file.functionScopes[1].variableDeclarations).to.be.length(1);
                            chai_1.expect(file.functionScopes[1].variableDeclarations[0]).to.deep.include({
                                lineIndex: 3,
                                name: 'age'
                            });
                            chai_1.expect(file.functionScopes[1].variableDeclarations[0].type).instanceof(IntegerType_1.IntegerType);
                            chai_1.expect(file.functionScopes[2].variableDeclarations).to.be.length(1);
                            chai_1.expect(file.functionScopes[2].variableDeclarations[0]).to.deep.include({
                                lineIndex: 7,
                                name: 'name'
                            });
                            chai_1.expect(file.functionScopes[2].variableDeclarations[0].type).instanceof(StringType_1.StringType);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('finds variable declarations inside of if statements', function () { return __awaiter(_this, void 0, void 0, function () {
                var scope;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    if true then\n                        theLength = 1\n                    end if\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            scope = file.getFunctionScopeAtPosition(vscode_languageserver_1.Position.create(3, 0));
                            chai_1.expect(scope.variableDeclarations[0]).to.exist;
                            chai_1.expect(scope.variableDeclarations[0].name).to.equal('theLength');
                            return [2 /*return*/];
                    }
                });
            }); });
            it('finds value from global return', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                   myName = GetName()\n                end sub\n\n                function GetName() as string\n                    return \"bob\"\n                end function\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes[0].variableDeclarations).to.be.length(1);
                            chai_1.expect(file.functionScopes[0].variableDeclarations[0]).to.deep.include({
                                lineIndex: 2,
                                name: 'myName'
                            });
                            chai_1.expect(file.functionScopes[0].variableDeclarations[0].type).instanceof(StringType_1.StringType);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('finds variable type from other variable', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                   name = \"bob\"\n                   nameCopy = name\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes[0].variableDeclarations).to.be.length(2);
                            chai_1.expect(file.functionScopes[0].variableDeclarations[1]).to.deep.include({
                                lineIndex: 3,
                                name: 'nameCopy'
                            });
                            chai_1.expect(file.functionScopes[0].variableDeclarations[1].type).instanceof(StringType_1.StringType);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('sets proper range for functions', function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, file.parse("\n                sub Main()\n                    getName = function()\n                        return \"bob\"\n                    end function\n                end sub\n            ")];
                        case 1:
                            _a.sent();
                            chai_1.expect(file.functionScopes).to.be.length(2);
                            chai_1.expect(file.functionScopes[0].bodyRange).to.eql(vscode_languageserver_1.Range.create(2, 0, 5, 16));
                            chai_1.expect(file.functionScopes[1].bodyRange).to.eql(vscode_languageserver_1.Range.create(3, 0, 4, 20));
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    }); });
    describe('getHover', function () {
        it('works for param types', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, hover;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub DoSomething(name as string)\n                    name = 1\n                    sayMyName = function(name as string)\n                    end function\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        hover = file.getHover(vscode_languageserver_1.Position.create(2, 24));
                        chai_1.expect(hover).to.exist;
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(2, 20, 2, 24));
                        //hover over the `name` parameter declaration
                        hover = file.getHover(vscode_languageserver_1.Position.create(1, 34));
                        chai_1.expect(hover).to.exist;
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(1, 32, 1, 36));
                        return [2 /*return*/];
                }
            });
        }); });
        //ignore this for now...it's not a huge deal
        it.skip('does not match on keywords or data types', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main(name as string)\n                end sub\n                sub as()\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        //hover over the `as`
                        chai_1.expect(file.getHover(vscode_languageserver_1.Position.create(1, 31))).not.to.exist;
                        //hover over the `string`
                        chai_1.expect(file.getHover(vscode_languageserver_1.Position.create(1, 36))).not.to.exist;
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds declared function', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, hover;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                function Main(count = 1)\n                    firstName = \"bob\"\n                    age = 21\n                    shoeSize = 10\n                end function\n            ")];
                    case 1:
                        file = _a.sent();
                        hover = file.getHover(vscode_languageserver_1.Position.create(1, 28));
                        chai_1.expect(hover).to.exist;
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(1, 25, 1, 29));
                        chai_1.expect(hover.contents).to.equal('function Main(count? as dynamic) as dynamic');
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds variable function hover in same scope', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, hover;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    sayMyName = sub(name as string)\n                    end sub\n\n                    sayMyName()\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        hover = file.getHover(vscode_languageserver_1.Position.create(5, 24));
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(5, 20, 5, 29));
                        chai_1.expect(hover.contents).to.equal('sub sayMyName(name as string) as void');
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds function hover in file scope', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, hover;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    sayMyName()\n                end sub\n\n                sub sayMyName()\n\n                end sub\n            ")];
                    case 1:
                        file = _a.sent();
                        hover = file.getHover(vscode_languageserver_1.Position.create(2, 25));
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(2, 20, 2, 29));
                        chai_1.expect(hover.contents).to.equal('sub sayMyName() as void');
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds function hover in context scope', function () { return __awaiter(_this, void 0, void 0, function () {
            var rootDir, program, mainFile, hover;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        rootDir = process.cwd();
                        program = new Program_1.Program({
                            rootDir: rootDir
                        });
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    sayMyName()\n                end sub\n            ")];
                    case 1:
                        mainFile = _a.sent();
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/lib.brs", "\n                sub sayMyName(name as string)\n\n                end sub\n            ")];
                    case 2:
                        _a.sent();
                        hover = mainFile.getHover(vscode_languageserver_1.Position.create(2, 25));
                        chai_1.expect(hover).to.exist;
                        chai_1.expect(hover.range).to.eql(vscode_languageserver_1.Range.create(2, 20, 2, 29));
                        chai_1.expect(hover.contents).to.equal('sub sayMyName(name as string) as void');
                        return [2 /*return*/];
                }
            });
        }); });
        it('handles mixed case `then` partions of conditionals', function () { return __awaiter(_this, void 0, void 0, function () {
            var mainFile;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    if true then\n                        print \"works\"\n                    end if\n                end sub\n            ")];
                    case 1:
                        mainFile = _a.sent();
                        chai_1.expect(mainFile.getDiagnostics()).to.be.lengthOf(0);
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    if true Then\n                        print \"works\"\n                    end if\n                end sub\n            ")];
                    case 2:
                        mainFile = _a.sent();
                        chai_1.expect(mainFile.getDiagnostics()).to.be.lengthOf(0);
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/source/main.brs", "\n                sub Main()\n                    if true THEN\n                        print \"works\"\n                    end if\n                end sub\n            ")];
                    case 3:
                        mainFile = _a.sent();
                        chai_1.expect(mainFile.getDiagnostics()).to.be.lengthOf(0);
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
//# sourceMappingURL=BrsFile.spec.js.map