/// <reference types="node" />
import { EventEmitter } from 'events';
import { CompletionItem, Hover, Position, Range } from 'vscode-languageserver';
import { FunctionScope } from '../FunctionScope';
import { Callable, Diagnostic, File, FileReference, FunctionCall } from '../interfaces';
import { Program } from '../Program';
export declare class XmlFile {
    pathAbsolute: string;
    /**
     * The absolute path to the file, relative to the pkg
     */
    pkgPath: string;
    program: Program;
    constructor(pathAbsolute: string, 
    /**
     * The absolute path to the file, relative to the pkg
     */
    pkgPath: string, program: Program);
    parentNameRange: Range;
    /**
     * The extension for this file
     */
    extension: string;
    ownScriptImports: FileReference[];
    getDiagnostics(): Diagnostic[];
    /**
     * The range of the entire file
     */
    fileRange: Range;
    parseDiagnistics: Diagnostic[];
    callables: Callable[];
    functionCalls: FunctionCall[];
    functionScopes: FunctionScope[];
    /**
     * The name of the component that this component extends
     */
    parentName: string;
    /**
     * The name of the component declared in this xml file
     */
    componentName: string;
    /**
     * Indicates if this file was processed by the program yet.
     */
    wasProcessed: boolean;
    parse(fileContents: string): Promise<void>;
    /**
     * Get the list of scripts imported by this component and all of its ancestors
     */
    getAllScriptImports(): FileReference[];
    /**
     * Get the list of scripts explicitly imported by this file.
     * This method excludes any ancestor imports
     */
    getOwnScriptImports(): FileReference[];
    /**
     * Determines if this xml file has a reference to the specified file (or if it's itself)
     * @param file
     */
    doesReferenceFile(file: File): boolean;
    /**
     * Get all available completions for the specified position
     * @param lineIndex
     * @param columnIndex
     */
    getCompletions(position: Position): {
        completions: CompletionItem[];
        includeContextCallables: boolean;
    };
    private getScriptImportAtPosition;
    private getScriptImportCompletions;
    /**
     * Scan the xml and find the range of the parent component's name in the `extends="ParentComponentName"` attribute of the component
     */
    findExtendsPosition(fullText: string): Range;
    emitter: EventEmitter;
    on(name: 'attach-parent', callback: (data: XmlFile) => void): any;
    on(name: 'detach-parent', callback: () => void): any;
    protected emit(name: 'attach-parent', data: XmlFile): any;
    protected emit(name: 'detach-parent'): any;
    parent: XmlFile;
    /**
     * Components can extend another component.
     * This method attaches the parent component to this component, where
     * this component can listen for script import changes on the parent.
     * @param parent
     */
    attachParent(parent: XmlFile): void;
    detachParent(): void;
    getHover(position: Position): Hover;
    getFunctionScopeAtPosition(position: Position, functionScopes?: FunctionScope[]): FunctionScope;
    dispose(): void;
}
