"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var path = require("path");
var sinonImport = require("sinon");
var vscode_languageserver_1 = require("vscode-languageserver");
var DiagnosticMessages_1 = require("../DiagnosticMessages");
var Program_1 = require("../Program");
var BrsFile_1 = require("./BrsFile");
var XmlFile_1 = require("./XmlFile");
var n = path.normalize;
describe('XmlFile', function () {
    var rootDir = process.cwd();
    var program;
    var sinon = sinonImport.createSandbox();
    var file;
    beforeEach(function () {
        program = new Program_1.Program({ rootDir: rootDir });
        file = new XmlFile_1.XmlFile(rootDir + "/components/MainComponent.xml", 'components/MainComponent.xml', program);
    });
    afterEach(function () {
        sinon.restore();
    });
    describe('parse', function () {
        it('does not include commented-out script imports', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\" extends=\"Scene\">\n                <script type=\"text/brightscript\" uri=\"ChildScene.brs\" />\n                <!--\n                        <script type=\"text/brightscript\" uri=\"ChildScene.brs\" />\n                    -->\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.ownScriptImports).to.be.lengthOf(1);
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds scripts when more than one per line', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\" extends=\"Scene\">\n                    <script type=\"text/brightscript\" uri=\"ChildScene1.brs\" /> <script type=\"text/brightscript\" uri=\"ChildScene2.brs\" /> <script type=\"text/brightscript\" uri=\"ChildScene3.brs\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.ownScriptImports).to.be.lengthOf(3);
                        chai_1.expect(file.ownScriptImports[0]).to.deep.include({
                            lineIndex: 3,
                            text: 'ChildScene1.brs',
                            columnIndexBegin: 58,
                            columnIndexEnd: 73
                        });
                        chai_1.expect(file.ownScriptImports[1]).to.deep.include({
                            lineIndex: 3,
                            text: 'ChildScene2.brs',
                            columnIndexBegin: 116,
                            columnIndexEnd: 131
                        });
                        chai_1.expect(file.ownScriptImports[2]).to.deep.include({
                            lineIndex: 3,
                            text: 'ChildScene3.brs',
                            columnIndexBegin: 174,
                            columnIndexEnd: 189
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds component names', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\" extends=\"ParentScene\">\n                <script type=\"text/brightscript\" uri=\"ChildScene.brs\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.parentName).to.equal('ParentScene');
                        chai_1.expect(file.componentName).to.equal('ChildScene');
                        return [2 /*return*/];
                }
            });
        }); });
        it('Adds error when no component is declared in xml', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("<script type=\"text/brightscript\" uri=\"ChildScene.brs\" />")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.parseDiagnistics).to.be.lengthOf(1);
                        chai_1.expect(file.parseDiagnistics[0].message).to.equal(DiagnosticMessages_1.diagnosticMessages.Xml_component_missing_component_declaration_1005().message);
                        return [2 /*return*/];
                }
            });
        }); });
        it('adds error when component does not declare a name', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component extends=\"ParentScene\">\n                <script type=\"text/brightscript\" uri=\"ChildScene.brs\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.parseDiagnistics).to.be.lengthOf(1);
                        chai_1.expect(file.parseDiagnistics[0]).to.deep.include({
                            message: DiagnosticMessages_1.diagnosticMessages.Component_missing_name_attribute_1006().message,
                            location: vscode_languageserver_1.Range.create(2, 16, 2, 26)
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('catches xml parse errors', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component 1extends=\"ParentScene\">\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.parseDiagnistics).to.be.lengthOf(1);
                        chai_1.expect(file.parseDiagnistics[0]).to.deep.include({
                            code: DiagnosticMessages_1.diagnosticMessages.Xml_parse_error_1008().code,
                            location: vscode_languageserver_1.Range.create(2, 27, 2, 27),
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds script imports', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abspath/components/cmp1.xml', 'components/cmp1.xml', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"Cmp1\" extends=\"Scene\">\n                    <script type=\"text/brightscript\" uri=\"pkg:/components/cmp1.brs\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.ownScriptImports.length).to.equal(1);
                        chai_1.expect(file.ownScriptImports[0]).to.deep.include({
                            sourceFile: file,
                            text: 'pkg:/components/cmp1.brs',
                            lineIndex: 3,
                            columnIndexBegin: 58,
                            columnIndexEnd: 82,
                            pkgPath: "components" + path.sep + "cmp1.brs"
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('throws an error if the file has already been parsed', function () { return __awaiter(_this, void 0, void 0, function () {
            var file, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abspath', 'relpath', null);
                        return [4 /*yield*/, file.parse("'a comment")];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, file.parse("'a new comment")];
                    case 3:
                        _a.sent();
                        chai_1.assert.fail(null, null, 'Should have thrown an exception, but did not');
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _a.sent();
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        }); });
        it('resolves relative paths', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abspath/components/cmp1.xml', 'components/cmp1.xml', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"Cmp1\" extends=\"Scene\">\n                    <script type=\"text/brightscript\" uri=\"cmp1.brs\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.ownScriptImports.length).to.equal(1);
                        chai_1.expect(file.ownScriptImports[0]).to.deep.include({
                            text: 'cmp1.brs',
                            pkgPath: "components" + path.sep + "cmp1.brs"
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it('finds correct position for empty uri in script tag', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abspath/components/cmp1.xml', 'components/cmp1.xml', null);
                        return [4 /*yield*/, file.parse("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"Cmp1\" extends=\"Scene\">\n                    <script type=\"text/brightscript\" uri=\"\" />\n                </component>\n            ")];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.ownScriptImports.length).to.equal(1);
                        chai_1.expect(file.ownScriptImports[0]).to.deep.include({
                            lineIndex: 3,
                            columnIndexBegin: 58,
                            columnIndexEnd: 58,
                        });
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('doesReferenceFile', function () {
        it('compares case insensitive', function () {
            var xmlFile = new XmlFile_1.XmlFile('absolute', 'relative', null);
            xmlFile.ownScriptImports.push({
                pkgPath: "components" + path.sep + "HeroGrid.brs",
                text: '',
                lineIndex: 1,
                sourceFile: xmlFile
            });
            var brsFile = new BrsFile_1.BrsFile('absolute', "components" + path.sep + "HEROGRID.brs", program);
            chai_1.expect(xmlFile.doesReferenceFile(brsFile)).to.be.true;
        });
    });
    describe('getCompletions', function () {
        it('formats completion paths with proper slashes', function () { return __awaiter(_this, void 0, void 0, function () {
            var scriptPath, xmlFile;
            return __generator(this, function (_a) {
                scriptPath = n('C:/app/components/component1/component1.brs');
                program.files[scriptPath] = new BrsFile_1.BrsFile(scriptPath, n('components/component1/component1.brs'), program);
                xmlFile = new XmlFile_1.XmlFile('component.xml', 'relative', program);
                xmlFile.ownScriptImports.push({
                    pkgPath: "",
                    text: '',
                    lineIndex: 1,
                    columnIndexBegin: 1,
                    columnIndexEnd: 1,
                    sourceFile: xmlFile
                });
                chai_1.expect(xmlFile.getCompletions(vscode_languageserver_1.Position.create(1, 1)).completions[0]).to.include({
                    label: 'components/component1/component1.brs',
                    kind: vscode_languageserver_1.CompletionItemKind.File
                });
                chai_1.expect(xmlFile.getCompletions(vscode_languageserver_1.Position.create(1, 1)).completions[1]).to.include({
                    label: 'pkg:/components/component1/component1.brs',
                    kind: vscode_languageserver_1.CompletionItemKind.File
                });
                return [2 /*return*/];
            });
        }); });
        it('returns empty set when out of range', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = new XmlFile_1.XmlFile('abs', 'rel', null);
                        return [4 /*yield*/, file.parse('')];
                    case 1:
                        _a.sent();
                        chai_1.expect(file.getCompletions(vscode_languageserver_1.Position.create(99, 99)).completions).to.be.empty;
                        return [2 /*return*/];
                }
            });
        }); });
        //TODO - refine this test once cdata scripts are supported
        it('prevents context completions entirely', function () { return __awaiter(_this, void 0, void 0, function () {
            var file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/components/Component1.brs", "")];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, program.addOrReplaceFile(rootDir + "/components/Component1.xml", "\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ParentScene\" extends=\"GrandparentScene\">\n                    <script type=\"text/brightscript\" uri=\"./Component1.brs\" />\n                </component>\n            ")];
                    case 2:
                        file = _a.sent();
                        chai_1.expect(program.getCompletions(file.pathAbsolute, vscode_languageserver_1.Position.create(1, 1))).to.be.empty;
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe('getAllScriptImports', function () {
        it('returns own imports', function () {
            var file = new XmlFile_1.XmlFile('file.xml', 'file.xml', null);
            var scriptImport = {
                text: 'some-import'
            };
            file.ownScriptImports.push(scriptImport);
            chai_1.expect(file.getAllScriptImports()).to.be.lengthOf(1);
        });
    });
    describe('findExtendsPosition', function () {
        it('works for single-line', function () {
            chai_1.expect(file.findExtendsPosition("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\" extends=\"BaseScene\">\n                </component>\n            ")).to.eql(vscode_languageserver_1.Range.create(2, 54, 2, 63));
        });
        it('works for multi-line', function () {
            chai_1.expect(file.findExtendsPosition("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\"\n                    extends=\"BaseScene\">\n                </component>\n            ")).to.eql(vscode_languageserver_1.Range.create(3, 29, 3, 38));
        });
        it('does not throw when unable to find extends', function () {
            file.findExtendsPosition("\n                <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n                <component name=\"ChildScene\">\n                </component>\n            ");
        });
    });
});
//# sourceMappingURL=XmlFile.spec.js.map