"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var DynamicType_1 = require("./DynamicType");
var FunctionType = /** @class */ (function () {
    function FunctionType(returnType) {
        this.returnType = returnType;
        /**
         * Determines if this is a sub or not
         */
        this.isSub = false;
        this.params = [];
    }
    FunctionType.prototype.setName = function (name) {
        this.name = name;
        return this;
    };
    FunctionType.prototype.addParameter = function (name, type, isRequired) {
        this.params.push({
            name: name,
            type: type,
            isRequired: isRequired === false ? false : true
        });
        return this;
    };
    FunctionType.prototype.isAssignableTo = function (targetType) {
        if (targetType instanceof DynamicType_1.DynamicType) {
            return true;
        }
        else if (targetType instanceof FunctionType) {
            //compare all parameters
            var len = Math.max(this.params.length, targetType.params.length);
            for (var i = 0; i < len; i++) {
                var myParam = this.params[i];
                var targetParam = targetType.params[i];
                if (!myParam || !targetParam || !myParam.type.isAssignableTo(targetParam.type)) {
                    return false;
                }
            }
            //compare return type
            if (!this.returnType || !targetType.returnType || !this.returnType.isAssignableTo(targetType.returnType)) {
                return false;
            }
            //made it here, all params and return type are equivalent
            return true;
        }
        else {
            return false;
        }
    };
    FunctionType.prototype.isConvertibleTo = function (targetType) {
        return this.isAssignableTo(targetType);
    };
    FunctionType.prototype.toString = function () {
        var e_1, _a;
        var paramTexts = [];
        try {
            for (var _b = __values(this.params), _c = _b.next(); !_c.done; _c = _b.next()) {
                var param = _c.value;
                paramTexts.push("" + param.name + (param.isRequired ? '' : '?') + " as " + param.type.toString());
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return (this.isSub ? 'sub' : 'function') + " " + this.name + "(" + paramTexts.join(', ') + ") as " + this.returnType.toString();
    };
    return FunctionType;
}());
exports.FunctionType = FunctionType;
//# sourceMappingURL=FunctionType.js.map